/********************************************************************************
*                                                                               *
*                                R e g U t i l                                  *
*                                                                               *
*********************************************************************************
* Copyright (C) 2001 by Sander Jansen.   All Rights Reserved.                   *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
*                                                                               *
********************************************************************************/	
#include"fx.h"

//
/*
THIS IS A WIN32 ONLY APPLICATION!!!

Notes:

--------key-----------------------------------------name---------------value---------
HKEY_CLASSES_ROOT\\.key                           default             *realkey*    
HKEY_CLASSES_ROOT\\.key                           Content Type        Mime Type      
HKEY_CLASSES_ROOT\\realkey                        default             description    
HKEY_CLASSES_ROOT\\realkey\\DefaultIcon           default             icon
HKEY_CLASSES_ROOT\\realkey\\Shell\\Open\\Command  default             application
HKEY_CLASSES_ROOT\\realkey\\Shell\\Play\\Command  default             application, plays file              

- Variations exist on the naming of Open and Play. So we search in the string for "open" or 
"play" instead of comparing. The string may contain "&Open with MSDev".

- If both play and open keys exsist we use play.
- If there is no description, but a mime type is available, we set the description to the mimetype.
- We add an '%s' to the application string if none exsist

Final string should be like this:

"applicationpath" + applicationflags + %s


Application specific flags that are removed:

- '/dde' - (Paint Shop Pro).
- %L - replaced by %s
- %1 - replaced by %s
*/

#ifdef WIN32
#define MAXNAME   200
#define MAXVALUE  2000


struct reginfo
{
  FXString app;
  FXString desc;
  FXString extension;
  FXString content;
  FXString icon_big;
  FXString icon_small;
};

//extracts path/executable name from application string
FXString GetDir(const FXString& file,FXint & n)
{
  register FXint i;
  FXbool whitespace=false;
  if(!file.empty()){
    i=0;
    if(isalpha((FXuchar)file[0]) && file[1]==':') i=2;
    if(file[i]=='\\') i++;
    n=i;
    while(file[i]){
      if (whitespace){
        if (file[i]=='\\'){
          n=i;
          whitespace=false;
          }
        i++;
        continue;
        }
      if (file[i]==' ') whitespace=true;
      n=i;
      i++;
      }
      return FXString(file.text(),n);
    }
  return FXString::null;
  }

//scans mime type...
void scan_mime(HKEY k,FXString & mime){
  FXuchar   value[MAXVALUE];
  FXchar    section[MAXNAME];
  DWORD     sectionsize=MAXNAME,i=0,namesize,valuesize=MAXVALUE,index;
  FILETIME  writetime;
  FXString tempstring;
 
  while( RegEnumValue(k,i,section,&sectionsize,NULL,NULL,value,&valuesize)==ERROR_SUCCESS ) {
    tempstring = section;
    if (tempstring == "Content Type") {
      mime = (FXchar*)value;
      return; 
      }
    i++;
    sectionsize=MAXNAME;valuesize=MAXVALUE;
    }
  }

//scans app key
void scan_app(HKEY k,FXString & app){
  FXchar   value[MAXVALUE];
  FXchar    section[MAXNAME];
  DWORD     sectionsize=MAXNAME,i=0,namesize,valuesize=MAXVALUE,index;
  FILETIME  writetime;
  FXString tempstring;

  HKEY l,m;
 
  if (RegOpenKeyEx(k,"shell",0,KEY_READ,&l) == ERROR_SUCCESS) {
    app = "";
    while(RegEnumKeyEx(l,i,section,&sectionsize,NULL,NULL,NULL,&writetime)==ERROR_SUCCESS){
      tempstring = section;
      tempstring = tempstring.lower();
      if (app.empty() && tempstring.findf("open") >=0 ){
        tempstring += "\\command";
        if (RegOpenKeyEx(l,tempstring.text(),0,KEY_READ,&m) == ERROR_SUCCESS) {
          if (RegQueryValue(m,NULL,value,(LONG*)&valuesize) == ERROR_SUCCESS){
            app = value;
            }
          RegCloseKey(m);
          }
        }
      if (tempstring.findf("play") >=0 ){
        tempstring += "\\command";
        if (RegOpenKeyEx(l,tempstring.text(),0,KEY_READ,&m) == ERROR_SUCCESS) {
          if (RegQueryValue(m,NULL,value,(LONG*)&valuesize) == ERROR_SUCCESS){
            app = value;
            break;
            }
          RegCloseKey(m);
          }
        }
      i++;
      sectionsize = MAXNAME;
      }
    RegCloseKey(l);
    }
  if (!app.empty())
  {
    FXint p,n;
    
   
    //remove all "
    while((p = app.findb('"')) >= 0) app.remove(p,1);
    app.trim();

    FXString newapp="";
     
    newapp += FXFile::enquote(GetDir(app,n));
    app.remove(0,n);
    if (app.findf("%") == -1) app.append(" %s");
    newapp += app;

    app = newapp;

    
   while((p = app.findf("/dde")) >=0) app.replace(p,4,"",0);
   while((p = app.findf("%1")) >= 0) app.replace(p,2,"%s");
   while((p = app.findf("%L")) >= 0) app.replace(p,2,"%s");
   
  }
}

//not working yet
void extract_and_save_icon(FXString iconkey,FXString &iconsmall,FXString &iconbig)
{
  FXString filename;
  FXString iconres;
  FXint    resvalue;

  filename = iconkey.before(',');
  iconres  = iconres.after(',');
  resvalue = FXIntVal(iconres);
  }


void scan()
{
  FXString lowercase,uppercase;
  reginfo  rinfo;
  FXchar    section[MAXNAME],value[MAXVALUE]; 
  DWORD     sectionsize,i,namesize,valuesize,index,type;
  FILETIME  writetime;
  FXString  name,newkey,iconkey;
  HKEY     skey,tkey;
 
  FXint     teller=0;
  i=0;
  sectionsize = MAXNAME;
  valuesize = MAXVALUE;

  FXRegistry * reg = new FXRegistry("Desktop");
  reg->read();
  reg->setAsciiMode(true);
  FXFileDict * filedict = new FXFileDict(NULL,reg);

       
  
  FXString ext;

  while(RegEnumKeyEx(HKEY_CLASSES_ROOT,i,section,&sectionsize,NULL,NULL,NULL,&writetime)==ERROR_SUCCESS){
    
    rinfo.app       = "";
    rinfo.extension = "";
    rinfo.content   = "";
    rinfo.extension = "";
    rinfo.desc      = "";

    name = section;
    if (name[0] == '.')
    {
      //assign extension
      rinfo.extension = name.after('.');
      //rinfo.extension = rinfo.extension.upper();
      
      if (RegOpenKeyEx(HKEY_CLASSES_ROOT,section,0,KEY_READ,&skey) == ERROR_SUCCESS)
      {
        scan_mime(skey,rinfo.content);
        
        
        valuesize = MAXVALUE;
        if (RegQueryValue(skey,NULL,value,(LONG*)&valuesize) == ERROR_SUCCESS)
        {
          valuesize = MAXVALUE;
          if (RegOpenKeyEx(HKEY_CLASSES_ROOT,value,0,KEY_READ,&skey) == ERROR_SUCCESS)
          {
            if (RegQueryValue(skey,NULL,value,(LONG*)&valuesize) == ERROR_SUCCESS)
            {
              rinfo.desc = value;
            }
            scan_app(skey,rinfo.app);
          }
        }

        if (rinfo.app.empty() &&
            rinfo.content.empty() &&
            rinfo.icon_big.empty() &&
            rinfo.icon_small.empty() &&
            rinfo.extension.empty() ) {
          continue;
        }

        if (rinfo.desc.empty() && !rinfo.content.empty())
          rinfo.desc = rinfo.content;

        if (rinfo.desc.empty())
          rinfo.desc = "Document";

       
        lowercase = rinfo.extension.lower();
        uppercase = rinfo.extension.upper();
        
        filedict->replace(lowercase.text(),FXStringFormat("%s;%s;;;%s",rinfo.app.text(),rinfo.desc.text(),rinfo.content.text()).text());
        filedict->replace(uppercase.text(),FXStringFormat("%s;%s;;;%s",rinfo.app.text(),rinfo.desc.text(),rinfo.content.text()).text());
        
      }
    }      
    i++;
    sectionsize = MAXNAME; 
  }
  reg->write();
  delete reg;
  delete filedict;
  
}







// Here we begin
int main(int argc,char *argv[]){
  fxmessage("FOX RegUtil\nCopyright (C) 2001 Sander Jansen\nThis software uses the FOX Platform Independent GUI Toolkit Library.\nThe FOX Library is Copyright (C) 1997,2001 Jeroen van der Zijp and is\navailable freely under GNU Library Public License at the following site:\nhttp://www.fox-toolkit.org/fox.html.");
  scan();
  return 0;
  }
#else


// Here we begin
int main(int argc,char *argv[]){
  fxerror("Platform not supported.\n");
  return 0;
  }

#endif

